package com.jpsoft.smart.lapi.handler;

import cn.hutool.json.JSONObject;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.lapi.dto.PersonVerification;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.PersonDeviceFilterLogService;
import com.jpsoft.smart.modules.base.service.PersonDeviceLogService;
import com.jpsoft.smart.modules.business.service.WorkAttendanceService;
import com.jpsoft.smart.modules.common.utils.LApiUtil;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.sys.entity.SysLog;
import com.jpsoft.smart.modules.sys.service.SysLogService;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.SocketAddress;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

@Slf4j
@Component
@ChannelHandler.Sharable
public class PersonVerificationHandler extends SimpleChannelInboundHandler<PersonVerification> {
    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private PersonDeviceLogService personDeviceLogService;

    @Autowired
    private PersonDeviceFilterLogService personDeviceFilterLogService;

    @Autowired
    private WorkAttendanceService workAttendanceService;

    @Autowired
    private SysLogService sysLogService;

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, PersonVerification personVerification) throws Exception {
        new Thread(()->{
            asyncWork(ctx,personVerification);
        }).start();
    }

    public void asyncWork(ChannelHandlerContext ctx,PersonVerification personVerification) {
        log.warn("===人员记录异步处理 begin===");

        JSONObject jsonObject = personVerification.getJsonObject();

        JSONObject faceJson = new JSONObject(jsonObject.getStr("FaceInfoList").substring(1,jsonObject.getStr("FaceInfoList").length()-1));

        Long timestamp = faceJson.getLong("Timestamp");

        if (timestamp.toString().length()<13){
            double timestampDouble = timestamp *  Math.pow(10, 13 - timestamp.toString().length());
            timestamp =new Double(timestampDouble).longValue() ;
        }

        //log.warn(String.valueOf(timestamp));
        Date date = new Date(timestamp);

        //设备序列号
        String deviceNo = jsonObject.getStr("DeviceCode");

        //人员温度信息
        BigDecimal temperature = faceJson.getBigDecimal("Temperature");

        //戴口罩状态（0：未开启口罩验证，1：未戴口罩，2：戴口罩）
        Integer maskFlag = faceJson.getInt("MaskFlag");

        //人员抓拍信息
        JSONObject faceImageJson = faceJson.getJSONObject("FaceImage");

        //匹配状态信息
        JSONObject libMatInfoListJson = new JSONObject();
        if (StringUtils.isNotBlank(jsonObject.getStr("LibMatInfoList").substring(1,jsonObject.getStr("LibMatInfoList").length()-1))){
            libMatInfoListJson = new JSONObject(jsonObject.getStr("LibMatInfoList").substring(1,jsonObject.getStr("LibMatInfoList").length()-1));
        }

        //匹配人员信息
        JSONObject matchPersonInfo = new JSONObject();
        if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchPersonInfo"))){
            matchPersonInfo = libMatInfoListJson.getJSONObject("MatchPersonInfo");
        }

        log.warn("人员记录写入");

        HashMap map = new HashMap<>();
        map.put("deviceNo",deviceNo);
        map.put("temperature",temperature);
        map.put("libMatInfoListJson",libMatInfoListJson);
        map.put("matchPersonInfo",matchPersonInfo);
        log.warn(map.toString());

        SysLog sysLog = new SysLog();
        sysLog.setPointcut(deviceNo);
        sysLog.setUrl("/LAPI/V1.0/System/Event/Notification/PersonVerification");
        sysLog.setData(map.toString());
        sysLog.setRemark("收到人脸识别信息");
        SocketAddress remoteAddress = ctx.channel().remoteAddress();
        sysLog.setRemoteIp(remoteAddress.toString());
        sysLog.setCreateTime(new Date());
        sysLogService.insert(sysLog);

        //上传人脸信息
        String faceImageUrl = "";

        try {
            if (StringUtils.isNotBlank(faceImageJson.getStr("Data"))){
                BASE64Decoder decoder = new BASE64Decoder();

                byte[] imgData = decoder.decodeBuffer(faceImageJson.getStr("Data"));

                for (int i = 0; i < imgData.length; ++i) {
                    if (imgData[i] < 0) {// 调整异常数据
                        imgData[i] += 256;
                    }
                }

                ByteArrayInputStream inputStream = new ByteArrayInputStream(imgData);


                faceImageUrl = OSSUtil.upload(ossConfig, "/devicePersonLog", faceImageJson.getStr("Name"), inputStream);
            }

        } catch (Exception e) {
            log.error("保存图片路径错误" + e.getMessage());
        }

        //记录人员记录
        personDeviceLogService.deviceInsertLog(deviceNo,temperature,faceImageJson,libMatInfoListJson,matchPersonInfo,date,maskFlag,faceImageUrl);

        //记录人员过滤记录
        personDeviceFilterLogService.deviceInsertFilterLog(deviceNo,temperature,faceImageJson,libMatInfoListJson,matchPersonInfo,date,maskFlag,faceImageUrl);

        //记录考勤信息
        try {
            final PersonInfo personInfo = LApiUtil.getPersonCode(libMatInfoListJson, matchPersonInfo);
            final BigDecimal fTemperature = temperature;
            final Date recordDate = date;

            if(personInfo.getId()!=null && personInfo.getId()!=0L) {
                workAttendanceService.punchIn(personInfo.getId(), fTemperature, recordDate);
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
        }

        log.warn("===人员记录异步处理 end===");
    }
}
